/*
    Render Frame
        あかつきみさき (みくちぃP)

    このスクリプトについて
        現在時間のフレームをpngに書き出すスクリプト.
        同じ階層にモジュールを組み込んだaepファイルを配置して実行してください.

    使用方法
        ファイル→スクリプト→スクリプトファイルを実行.

    動作環境
        Adobe After Effects CC 2014以上

    バージョン情報
        2018/07/01 Ver 1.0.0 Release
*/

(function () {
    var SAVE_SUPPORT_EXTENTION = ["PNG files:*.png", "All files:*.*"];

    var aetFiles = new File($.fileName).parent.getFiles(/.aet$|.aep$/);
    if (aetFiles == []) {
        alert("出力モジュールプロジェクトファイルが見つかりませんでした", "Render Frame");
        return 0;
    }

    var actComp = app.project.activeItem;
    if (actComp == null || !(actComp instanceof CompItem)) {
        return;
    }

    aetFiles = aetFiles.sort().reverse();

    var importedItem = app.project.importFile(new ImportOptions(aetFiles[0]));

    var enabledModule = false;
    for (var i = 1; i <= app.project.renderQueue.item(app.project.renderQueue.numItems).outputModule(1).templates.length; i++) {
        if (app.project.renderQueue.item(app.project.renderQueue.numItems).outputModule(1).templates[i] == "PNG_Alpha") {
            enabledModule = true;
        }
    }

    if (!enabledModule) {
        app.project.renderQueue.item(app.project.renderQueue.numItems).outputModule(1).saveAsTemplate("PNG_Alpha");
    }
    app.project.renderQueue.item(app.project.renderQueue.numItems).remove();
    importedItem.remove();

    var curFile = app.project.file;
    if (curFile == null) {
        curFile = new File(Folder.desktop.fullName + "/" + actComp.name + ".png");
    }

    var outFile = new File(curFile.parent.fullName + "/" + actComp.name).saveDlg("保存先", SAVE_SUPPORT_EXTENTION);
    if (outFile == null) {
        return 0;
    }

    app.beginUndoGroup("Render Frame");
    app.beginSuppressDialogs();

    var renderEnabled = [];
    for (var i = 1; i <= app.project.renderQueue.items.length; i++) {
        renderEnabled[renderEnabled.length] = app.project.renderQueue.item(i).render;
        app.project.renderQueue.item(i).render = false;
    }

    var queue = app.project.renderQueue.items.add(actComp);
    queue.outputModule(1).applyTemplate("PNG_Alpha");
    queue.outputModule(1).file = new File(outFile.parent.fullName + "/" + outFile.name.replace(/.png$/, "") + "_[####].png");
    queue.timeSpanStart = actComp.time;
    queue.timeSpanDuration = actComp.frameDuration;

    app.project.renderQueue.render();
    actComp.openInViewer();
    queue.remove();

    for (var i = 0; i < renderEnabled.length; i++) {
        app.project.renderQueue.item(i + 1).render = renderEnabled[i];
    }

    app.endSuppressDialogs(false);
    app.endUndoGroup();
    return 0;
})();
